package Bundle::Text::Query::BuildSQL;

$VERSION = "0.03";

1;

__END__

=head1 NAME

Bundle::Text::Query::BuildSQL - A bundle to install related modules

=head1 SYNOPSIS

 perl -MCPAN -e 'install Bundle::Text::Query::BuildSQL'

=head1 CONTENTS

DBI 1.06                   - Database independent interface for Perl

Text::Query 0.06 	   - Parse and match Boolean expression

=head1 DESCRIPTION

This bundle defines all reqreq modules.

=head1 AUTHOR

Loic Dachary <loic@senga.org>

=cut
